﻿namespace HeuristicLab.BenchmarkSuite.Problems {
  using System.Linq;

  using HeuristicLab.Common;
  using HeuristicLab.Core;
  using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

  [StorableClass]
  public class Data : NamedItem {

    public Data() { }

    [StorableConstructor]
    protected Data(bool deserializing) : base(deserializing) { }

    public Data(Data origin, Cloner cloner) : base(origin, cloner) {
      OriginalTrainingCount = origin.OriginalTrainingCount;
      OriginalTestCount = origin.OriginalTestCount;
      BestResult = origin.BestResult;
      WorstResult = origin.WorstResult;
      InputArgumentTypes = origin.InputArgumentTypes;
      OutputArgumentTypes = origin.OutputArgumentTypes;

      if (origin.Examples != null)
        Examples = origin.Examples.Select(e => (Example)e.Clone(cloner)).ToArray();

      EnabledDataTypes = origin.EnabledDataTypes;
    }

    [Storable]
    public int OriginalTrainingCount { get; set; }
    [Storable]
    public int OriginalTestCount { get; set; }
    [Storable]
    public double BestResult { get; set; }
    [Storable]
    public double WorstResult { get; set; }
    [Storable]
    public ExampleArgumentType[] InputArgumentTypes { get; set; }
    [Storable]
    public ExampleArgumentType[] OutputArgumentTypes { get; set; }
    public int TotalArgumentCount { get { return InputArgumentTypes.Length + OutputArgumentTypes.Length; } }
    [Storable]
    public Example[] Examples { get; set; }
    [Storable]
    public DataTypes EnabledDataTypes { get; set; }

    /// <summary>
    /// Max Size” gives the maximum number of instructions that can appear in an individual’s genome
    /// </summary>
    [Storable]
    public int MaxSize { get; set; }

    /// <summary>
    /// “Eval Limit” is the number of steps of the Push interpreter that are executed before stopping a
    /// program’s execution; programs halted in this way may still achieve good results if they print or
    /// return results before they are stopped
    /// </summary>
    [Storable]
    public int EvalLimit { get; set; }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new Data(this, cloner);
    }
  }
}
